/*
 * $Id: attractmanager.c,v 1.4 2003/02/09 07:34:15 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Attraction(score/title/gameover) manager.
 *
 * @version $Revision: 1.4 $
 */
//#include "SDL.h"
#include <stdlib.h>
#include <stdio.h>
#include "noiz2sa.h"
#include "screen.h"
#include "letterrender.h"
#include "attractmanager.h"
#include "bonus.h"
#include "brgmng_mtd.h"
#include "soundmanager.h"
#include "degutil.h"
#include "colors.h"
#include "background.h"

int score;
static int nextExtend, neAdd;
static int dsc;
int left, stage;
static int ssSc;
static int hsScene, hsScSc, hsOfs;

static HiScore hiScore;

#define PREF_FILE "noiz2sa.prf"
#define DEFAULT_HISCORE 100000
#define DEFAULT_SCENE_HISCORE 10000
#define PANEL_OFFSET 160 
static void initHiScore() {
  int i, j;
  for ( i=0 ; i<STAGE_NUM ; i++ ) {
    hiScore.stageScore[i] = DEFAULT_HISCORE;
    for ( j=0 ; j<SCENE_NUM ; j++ ) {
      hiScore.sceneScore[i][j] = DEFAULT_SCENE_HISCORE;
    }
  }
  for ( i=0 ; i<ENDLESS_STAGE_NUM ; i++ ) {
    hiScore.stageScore[i+STAGE_NUM] = DEFAULT_HISCORE;
  }
  hiScore.stage = 0;
}

// Load preference.
void loadPreference() {
    initHiScore(); 

  FILE *fp;
  int i, j;
  int version;
  if ( NULL == (fp = fopen(PREF_FILE,"rb")) ) {
    initHiScore();
    return;
  }
  version = getw(fp);
  if ( version != VERSION_NUM ) {
    initHiScore();
    return;
  }
  for ( i=0 ; i<STAGE_NUM ; i++ ) {
    hiScore.stageScore[i] = getw(fp);
    for ( j=0 ; j<SCENE_NUM ; j++ ) {
      hiScore.sceneScore[i][j] = getw(fp);
    }
  }
  for ( i=0 ; i<ENDLESS_STAGE_NUM ; i++ ) {
    hiScore.stageScore[i+STAGE_NUM] = getw(fp);
  }
  hiScore.stage = getw(fp);
  fclose(fp); 
}

// Save preference.
void savePreference() {

  FILE *fp;
  int i, j;
  if ( NULL == (fp = fopen(PREF_FILE,"wb")) ) return;
  putw(VERSION_NUM, fp);
  for ( i=0 ; i<STAGE_NUM ; i++ ) {
    putw(hiScore.stageScore[i], fp);
    for ( j=0 ; j<SCENE_NUM ; j++ ) {
      putw(hiScore.sceneScore[i][j], fp);
    }
  }
  for ( i=0 ; i<ENDLESS_STAGE_NUM ; i++ ) {
    putw(hiScore.stageScore[i+STAGE_NUM], fp);
  }
  putw(hiScore.stage, fp);
  fclose(fp);

}

void initGameState(int stg) {
  score = 0; ssSc = 0;
  nextExtend = 200000;
  neAdd = 300000;
  dsc = -1;
  left = 2;
  hiScore.stage = stage = stg;
  hsScene = -1;
  drawRPanel();
}

void addScore(int s) {
  score += s;
  if ( score >= nextExtend ) {
    nextExtend += neAdd;
    neAdd = 500000;
    if ( extendShip() ) {
      playChunk(6);
    }
  }
}

int extendShip() {
  if ( left > 8 ) return 0;
  left++;
  drawRPanel();
  return 1;
}

int decrementShip() {
  left--;
  drawRPanel();
  if ( left < 0 ) return 1;
  return 0;
}

void addLeftBonus() {
  nextExtend = 999999999;
  addScore(left*100000);
}

void setClearScore() {
  int rss = hiScore.sceneScore[stage][scene];
  int ss = score - ssSc;
  hsScene = scene;
  hsOfs = ss - rss;
  hsScSc = ss;
  if ( ss > rss ) {
    hiScore.sceneScore[stage][scene] = ss;
  }
  drawRPanel();
  ssSc = score;
}

void setHiScore() {
  if ( score > hiScore.stageScore[stage] ) {
    hiScore.stageScore[stage] = score;
  }
}

void showScore() {
  dsc = -1;
}

void drawScore() {
  if ( dsc == score ) return;
  dsc = score;
  clearLPanel();
  drawNumCenter(bonusScore, 39, 495, 7, NUMBOX_COLOR1, NUMBOX_COLOR2);
  drawNumCenter(score, 162 , 497, 10, NUMBOX_COLOR1, NUMBOX_COLOR2);
}

#define SCENE_STAT_SIZE 7 

void drawRPanel() {
  int y,x;
  char *str = "LEFT";
  clearRPanel();
  if ( left >= 0 ) {
    drawString(str, PANEL_OFFSET+ 86, 496, 8, 2, NUMBOX_COLOR1, NUMBOX_COLOR2);
    drawLetter(left, PANEL_OFFSET+ 148, 496, 8, 2, LIVES_COLOR1, LIVES_COLOR2);
  } else {
    drawString("-----", PANEL_OFFSET+ 86, 496, 8, 2, NUMBOX_COLOR1, NUMBOX_COLOR2);
  } 
  y = 497;
  x=34+PANEL_OFFSET;
  if ( !endless ) {
    y = drawNumCenter(stage+1, x , y, 7, NUMBOX_COLOR1, NUMBOX_COLOR2);
    drawLetter(38, x+9, y, 7, 2,  NUMBOX_COLOR1, NUMBOX_COLOR2);
     x += 20;
	
    if ( scene >= 10 ) {
	x+=9;
    //  drawLetter('E'-'A'+10, x , y, 7, 2, NUMBOX_COLOR1, NUMBOX_COLOR2);
     // return;
    }
  }
  drawNumCenter(scene+1, x , y, 7, NUMBOX_COLOR1, NUMBOX_COLOR2);
}

void drawSceneStats() {
 int y=10, x=10;

  if ( hsScene >= 0 ) {
    x = drawNumLeft2Right(stage+1, x, y, SCENE_STAT_SIZE,  NUMBOX_COLOR1, NUMBOX_COLOR2);
    x += SCENE_STAT_SIZE*1.7f;
    drawLetter(38, x, y, SCENE_STAT_SIZE, 2,   NUMBOX_COLOR1, NUMBOX_COLOR2);
    x = drawNumLeft2Right(hsScene+1, x, y, SCENE_STAT_SIZE,  NUMBOX_COLOR1, NUMBOX_COLOR2);
    x += SCENE_STAT_SIZE*1.7f*2;
    x = drawNumLeft2Right(hsScSc, x, y, SCENE_STAT_SIZE, 16* NUMBOX_COLOR1, NUMBOX_COLOR2);
    x += SCENE_STAT_SIZE*1.7f;
    if ( hsOfs >= 0 ) {
      drawLetter(39, x, y, SCENE_STAT_SIZE, 2,   NUMBOX_COLOR2, NUMBOX_COLOR2);
      x += SCENE_STAT_SIZE*1.7f;
      drawNumLeft2Right(hsOfs, x, y, SCENE_STAT_SIZE,NUMBOX_COLOR2, NUMBOX_COLOR2);
    } else {
      drawLetter(38, x, y, SCENE_STAT_SIZE, 2,  16*4-12, 16*4-3);
      x += SCENE_STAT_SIZE*1.7f;
      drawNumLeft2Right(-hsOfs, x, y, SCENE_STAT_SIZE, 16*4-12, 16*4-3);
    }
  }
}

#define STG_BOX_SIZE 40
#define STG_BOX_NUM 15

static int stageX[STG_BOX_NUM], stageY[STG_BOX_NUM];

void initAttractManager() {
  int i, j, x, y, s;
  y = LAYER_HEIGHT/3+STG_BOX_SIZE/2;
  s = 0;
  setColors();
  for ( i=0 ; i<6 ; i++, y += STG_BOX_SIZE*1.2f ) {
    //x = STG_BOX_SIZE/2+STG_BOX_SIZE/2;
     x=60;
    switch ( i ) {
    case 0:
    case 1:
    case 2:
    case 3:
      for ( j=0 ; j<=i ; j++, s++, x+=STG_BOX_SIZE*1.2f ) {
	stageX[s] = x; stageY[s] = y;
      }
      break;
    case 4:
      for ( j=0 ; j<=2 ; j++, s++, x+=STG_BOX_SIZE*1.2f ) {
	stageX[s] = x; stageY[s] = y;
      }
      x += STG_BOX_SIZE*1.2f;
      stageX[s] = x; stageY[s] = y;
      s++;
      break;
    case 5:
      y += STG_BOX_SIZE/3;
      stageX[s] = x; stageY[s] = y;
      break;
    }
  }
}

static int titleCnt;
static int slcStg;
static int mnp;

int initTitleAtr() {
  stopMusic();
  titleCnt = 0;
  slcStg = hiScore.stage;
  mnp = 0;
  return slcStg;
}

void drawTitle() {
  int i;
  for ( i=0 ; i<7 ; i++ ) {
	drawSprite(i, 512-56,(i*46+2) );
  }
}

static int stgMv[STAGE_NUM+ENDLESS_STAGE_NUM+1][4] = {
  {0, 0, 1, 0}, 
  {-1, 1, 2, 0}, {0, 0, 2, -1}, 
  {-2, 1, 3, 0}, {-2, 1, 3, -1}, {0, 0, 3, -1},
  {-3, 1, 4, 0}, {-3, 1, 4, -1}, {-3, 1, 4, -1}, {0, 0, 3, -1},
  {-4, 1, 4, 0}, {-4, 1, 3, -1}, {-4, 1, 2, -1},                {0, 0, 1, -1},
  {-4, 0, 0, 0},
};

void moveTitleMenu() {
  int pad = getPadState();
  int btn = getButtonState();
  int p = -1;
  int sm;
  if ( pad & PAD_DOWN ) {
    p = 2;
    playChunk(0);
  } else if ( pad & PAD_UP ) {
    p = 0;
    playChunk(0);
  } else if ( pad & PAD_RIGHT ) {
    p = 1;
    playChunk(0);
  } else if ( pad & PAD_LEFT ) {
    p = 3;
    playChunk(0);
  } else if ( btn == 0 ) {
    mnp = 1;
  }
  if ( mnp && p >= 0 ) {
    mnp = 0;
    sm = stgMv[slcStg][p];
    slcStg += sm;
    if ( sm != 0 ) {
      initTitleStage(slcStg);
	initBackground();
	setStageBackground(slcStg%5+1);
        setBG(8+slcStg%5);

      playMusic(slcStg+1); 
    }
    titleCnt = 16;
  }
  if ( mnp && (btn & PAD_BUTTON1) ) {
    if ( slcStg == STAGE_NUM+ENDLESS_STAGE_NUM ) {
      quitLast();
      return;
    }
    hiScore.stage = slcStg;
    initGame(slcStg);
  }
  titleCnt++;
}

void drawTitleMenu() {
  int i;
  char *stgChr = "STAGE";
  char *endlessChr = "ENDLESS";
  char *hardChr = "HARD";
  char *extChr = "EXTREME";
  char *insChr = "INSANE";
  char *quitChr = "QUIT";
  for ( i=0 ; i<STG_BOX_NUM ; i++ ) {
    if ( i == slcStg ) {
      int sz = STG_BOX_SIZE+6+sctbl[(titleCnt*16)&(DIV-1)]/24;
      drawBox(stageX[i], stageY[i], sz, sz, SELECT_COLOR1, SELECT_COLOR2);
      if ( i < STAGE_NUM ) {
	drawStringBuf(stgChr, 180, 80, 12, 2,  NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	drawNumCenter(i+1, 308, 80, 12,  NUMBOXCOLOR1,  NUMBOXCOLOR2);
      } else {
	switch ( i ) {
	case 10:
	  drawStringBuf(endlessChr, 188, 80, 12, 2,  NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	  break;
	case 11:
	  drawStringBuf(endlessChr, 93, 80, 12, 2,  NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	  drawStringBuf(hardChr, 248, 80, 12, 2,  NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	  break;
	case 12:
	  drawStringBuf(endlessChr, 36, 80, 12, 2,  NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	  drawStringBuf(extChr, 190, 80, 12, 2,  NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	  break;
	case 13:
	  drawStringBuf(endlessChr, 56, 80, 12, 2,  NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	  drawStringBuf(insChr, 210, 80, 12, 2,  NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	  break;
	case 14:
	  drawStringBuf(quitChr, 230, 80, 12, 2,  NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	  break;
	}
      }
      if ( i < STAGE_NUM+ENDLESS_STAGE_NUM ) {
	drawNumCenter(hiScore.stageScore[i], 308, 112, 12,  NUMBOXCOLOR1,  NUMBOXCOLOR2);
      }
    }
    drawBox(stageX[i], stageY[i], STG_BOX_SIZE, STG_BOX_SIZE, NUMBOXCOLOR1, NUMBOXCOLOR2 );
    if ( i < 9 ) {
      drawNumCenter(i+1, stageX[i], stageY[i], 12, NUMBOXCOLOR1,  NUMBOXCOLOR2);
    } else {
      switch ( i ) {
      case 9:
	drawNumCenter(10, stageX[i]+8, stageY[i], 12, NUMBOXCOLOR1,  NUMBOXCOLOR2);
	break;
      case 10:
	drawLetterBuf('E'-'A'+10, stageX[i], stageY[i], 12, 2, NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	break;
      case 11:
	drawLetterBuf('E'-'A'+10, stageX[i]-8, stageY[i], 12, 2, NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	drawLetterBuf('H'-'A'+10, stageX[i]+8, stageY[i], 12, 2, NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	break;
      case 12:
	drawLetterBuf('E'-'A'+10, stageX[i]-8 , stageY[i], 12, 2, NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	drawLetterBuf('E'-'A'+10, stageX[i]+8, stageY[i], 12, 2, NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	break;
      case 13:
	drawLetterBuf('E'-'A'+10, stageX[i]-8 , stageY[i], 12, 2, NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	drawLetterBuf('I'-'A'+10, stageX[i]+8 , stageY[i], 12, 2, NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	break;
      case 14:
	drawLetterBuf('Q'-'A'+10, stageX[i], stageY[i], 12, 2, NUMBOXCOLOR1,  NUMBOXCOLOR2,  0);
	break;
      }
    }
  }
}

static int goCnt;

void initGameoverAtr() {
  goCnt = 0;
  mnp = 0;
  fadeMusic();
}

void moveGameover() {
  int btn = getButtonState();
  if ( goCnt > 900 || (goCnt > 128 && mnp && (btn & PAD_BUTTON1)) ) {
    setHiScore();
    initTitle();
    return;
  }
  if ( btn == 0 ) {
    mnp = 1;
  }
  goCnt++;
}

void drawGameover() {
  char *goChr = "GAME OVER";
  int y;
  if ( goCnt < 128 ) {
    y = LAYER_HEIGHT/3*goCnt/128;
  } else {
    y = LAYER_HEIGHT/3;
  }
  drawStringBuf(goChr, 24 , y, 20, 2, GAMEOVER_COLOR, PAUSE_COLOR2,  0);
}

static int scCnt;

void initStageClearAtr() {
  scCnt = 0;
  mnp = 0;
  fadeMusic();
}

void moveStageClear() {
  int btn = getButtonState();
  if ( scCnt > 900 || (scCnt > 128 && mnp && (btn & PAD_BUTTON1)) ) {
    setHiScore();
    initTitle();
    return;
  }
  if ( btn == 0 ) {
    mnp = 1;
  }
  scCnt++;
}

void drawStageClear() {
  char *scChr = "STAGE CLEAR";
  int y;
  if ( scCnt < 128 ) {
    y = LAYER_HEIGHT - LAYER_HEIGHT/3*2*scCnt/128;
  } else {
    y = LAYER_HEIGHT/3;
  }
  drawStringBuf(scChr, 24 , y, 16, 2, STAGECLEAR_COLOR, PAUSE_COLOR2,  1);
}

static int psCnt = 0;

void movePause() {
  psCnt++;
}

void drawPause() {
  char *psChr = "PAUSE";
  if ( (psCnt&63) < 32 ) {
    drawStringBuf(psChr, 92 , LAYER_HEIGHT/3, 20, 2, PAUSE_COLOR1, PAUSE_COLOR2,  0);
  }
}
